#    Kye - classic puzzle game
#    Copyright (C) 2005, 2006 Colin Phipps <cph@moria.org.uk>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import pygtk
pygtk.require('2.0')
from gtk.gdk import keyval_from_name, SHIFT_MASK

class KMoveInput:
    def __init__(self):
        self.recordto = None
        self.clear()

    def clear(self):
	self.heldkeys = []
	self.keyqueue = []
	self.mousemoving = None
	self.currentmouse = None

    keymap = {
	keyval_from_name('Left')	: ("rel",-1, 0),
	keyval_from_name('Right')	: ("rel", 1, 0),
	keyval_from_name('Up')	: ("rel", 0,-1),
	keyval_from_name('Down')	: ("rel", 0, 1),
	keyval_from_name('KP_4') : ("rel",-1, 0),
	keyval_from_name('KP_6') : ("rel", 1, 0),
	keyval_from_name('KP_2') : ("rel", 0, 1),
	keyval_from_name('KP_8') : ("rel", 0,-1),
	keyval_from_name('KP_1') : ("rel",-1, 1),
	keyval_from_name('KP_3') : ("rel", 1, 1),
	keyval_from_name('KP_7') : ("rel",-1,-1),
	keyval_from_name('KP_9') : ("rel", 1,-1),
	keyval_from_name('KP_Left')	: ("rel",-1, 0),
	keyval_from_name('KP_Right')	: ("rel", 1, 0),
	keyval_from_name('KP_Down')	: ("rel", 0, 1),
	keyval_from_name('KP_Up')	: ("rel", 0,-1),
	keyval_from_name('KP_End')	: ("rel",-1, 1),
	keyval_from_name('KP_Page_Down') : ("rel", 1, 1),
	keyval_from_name('KP_Home')	: ("rel",-1,-1),
	keyval_from_name('KP_Page_Up')	: ("rel", 1,-1),
	}

    def key_press_event(self, widget, event):
	try:
            pressedkey =  KMoveInput.keymap[event.keyval]
            # If this key is not already pressed, then it's a new press and so we want to move at least one square
            #  and remember that it is held down.
            if pressedkey not in self.heldkeys:
                self.keyqueue.append(pressedkey)

                # There is no way at this point to know if this is a key press or a key hold. So we'll have a 'delay' which causes us to wait a few ticks before considering the key held and reacting to it again.
                # But have SHIFT as a way for the user to tell us that it's a hold.
                if event.state & SHIFT_MASK == 0:
                    self.__delay = 1 # wait 1 tic
                self.heldkeys.append(pressedkey)
	except KeyError, e:
	    return

    def key_release_event(self, widget, event):
	try:
	    self.heldkeys.remove(KMoveInput.keymap[event.keyval])
	except KeyError, e:
	    return
        except ValueError, e:
            return

    def mouse_motion_event(self, x, y):
        self.currentmouse = ("abs",x,y)

    def button_press_event(self, button, x, y):
	if button == 1:
	    self.mousemoving = True

    def button_release_event(self, button, x, y):
	if button == 1:
	    self.mousemoving = False

    def __get_move(self):
        # If there are key presses in the queue, use them first
	if len(self.keyqueue) > 0:
	    k = self.keyqueue[0]
	    self.keyqueue = self.keyqueue[1:]
	    return k
        # Then, if the mouse is pressed, do it
	if self.mousemoving and self.currentmouse:
	    return self.currentmouse
        # Finally, if any keys are held, use the most recently pressed
        if len(self.heldkeys) > 0:
            if self.__delay <= 0:
                return self.heldkeys[-1]
            self.__delay = self.__delay - 1
        # No action
        return None

    def record_to(self, stream):
        if self.recordto != None and self.recordto != stream:
            try:
                self.recordto.close()
            except IOerror, e:
                print "error closing recording"
        self.recordto = stream

    def get_move(self):
        m = self.__get_move()
        if self.recordto != None:
            if m != None:
                self.recordto.write("\t".join([str(i) for i in m]))
            self.recordto.write("\n")
        return m

class KyeRecordedInput:
    def __init__(self, stream):
        self.__s = stream

    def get_move(self):
        l = self.__s.readline()
        l = l.rstrip()
        if len(l) == 0:
            return None
        s = l.split("\t")
        m = [s[0],int(s[1]),int(s[2])]
        return m
